/****************************************************************
**
**    Mandel Ultimativ  (for KDE)
**
****************************************************************/

// Megainclude
#include <stdio.h>
#include <qapp.h>
#include <qpushbt.h>
#include <qfont.h>
#include <qlabel.h>
#include <qcolor.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qwindow.h>
#include <qfiledlg.h>
#include <qfile.h>
#include <kapp.h>
#include <ktopwidget.h>
#include <kkeyconf.h>
//#include <klocale.h>
#include "basefrac.h"
#include "kmandel.h"
#include "mandelbrot.h"
#include "julia.h"
#include "l_mandel.h"
#include "l_julia.h"
#include "invmandel.h"
#include "ginger.h"
#include "galaxy.h"
#include "hop.h"
#include "chip.h"
#include "aboutdlg.h"
#include "dlgfractype.h"
#include "dlgfracparams.h"
#include "fracglobals.h"
#include "mandelapp.h"
#include "dlgpicset.h"
#include "dlgeditpal.h"
#include "dlgzoomsettings.h"
#include "parameter.h"
#include "dlgfilesettings.h"
#include "mandel_n.h"
#include "julia_n.h"
#include "triple.h"
#include "henon.h"
#include "barnsja.h"
#include "barnsb.h"
#include "barnsc.h"
#include "orbit3.h"
#include "wankel.h"
#include "quadrat.h"
#include "liapi.h"
#include "star.h"
#include "lorenz.h"
#include "genetic.h"
#include "pheonix.h"
#include "pocken.h"
#include "kmandel.moc"
#define UL UpdateInterval
#define TRS klocale->translate

#define MID_FILE     100   // Men File
#define MID_FILENEW  101   // File - New

KMandel *kmand; 
MandelApp* MApp;
int StartUp;    // Startup-Flag for Toolbar-Positioning-Problem....

DrawWidget::DrawWidget( QWidget* parent=0, const char* name=0, WFlags f=0)
        : QWindow( parent, name, f )
{

   PicSizeX = 320;
   PicSizeY = 240;
   Berechnung = FALSE;   // keine Berechnung
   ImageReady = FALSE;   // kein Bild
   Resized    = FALSE;
   ZoomMode   = ZOOM_NONE;
   ZoomFactor = 5.0;
   ZoomFactorAnimation = 1.03;
   ZoomAnimationPics = 5;
   ColorDeep  = 8;       // 256 Farben
   NumColors  = 256;//xFFFFFFFF;
   SaveColorMapBool = FALSE;
   UpdateInterval = 50;
   UpdateLevel = 3;
   UpdateBool = TRUE;
   HiColorMode = 1;
   CalcMethod = CM_RECT;

   resize(PicSizeX,PicSizeY);

   Image = new QImage(PicSizeX, PicSizeY, ColorDeep, NumColors,QImage::BigEndian);
   Pix = new QPixmap();
//fprintf(stdout,"ImageBytes=%d\n",Image->numBytes());
   // Farbpalette setzen
   Image->setNumColors(256);       // Anzahl der Farben
   CreatePalette();
   InitPalette();

   Paint = new QPainter();
   Fractal = new MandelBrot(); //BaseFract();
   Fractal->Init(PicSizeX, PicSizeY);

   connect(this, SIGNAL(FertigSig()), SLOT(Fertig()));
}

void DrawWidget::SetPicSize(int SizeX, int SizeY)
{ 
   PicSizeX = SizeX;
   PicSizeY = SizeY;
   Fractal->SetSize(SizeX, SizeY);
   //ImageReady = FALSE;
   //repaint(TRUE);  // neue Berechnung erzwingen
}

void DrawWidget::resizeEvent(QResizeEvent*)
{
QImage* ImgTmp, *ImgOld;

//   fprintf(stdout, "DrawWidget::ResizeEvent()  x= %d y=%d\n",width(),height());
   AbortCalc();
   SetPicSize(width(),height());  // neue Gre
   ImgTmp = new QImage(PicSizeX, PicSizeY, ColorDeep, NumColors,QImage::BigEndian);
   ImgOld = Image;
   Image = ImgTmp;
   delete ImgOld;
   //Image = new QImage(PicSizeX, PicSizeY, ColorDeep, NumColors,QImage::BigEndian);
   InitPalette();
   Image->fill(0);
}

void DrawWidget::paintEvent(QPaintEvent*)
{
   //fprintf(stdout, "DrawWidget::paintEvent\n");
   //fprintf(stdout, "Image x=%d, y=%d\n",Image->width(), Image->height());
   //fprintf(stdout, "Window x=%d, y=%d\n",PicSizeX, PicSizeY);

   if (ImageReady) {         // das Bild ist fertig, sofort anzeigen
      //fprintf(stdout, "Repaint: Image ready\n");
      Paint->begin(this);
      //Pix = new QPixmap();
      Pix->convertFromImage(*Image, QPixmap::Auto);
      Paint->drawPixmap(0, 0, *Pix);
      //delete Pix;
   
      Paint->end();   
   } else
      CalcFractal(); 
   if (ImageReady)
      emit Fertig();
}

void DrawWidget::mousePressEvent (QMouseEvent* mev) 
{
   //fprintf(stdout,"mousePressEvent\n");
   if (mev->button() == LeftButton) {
      //fprintf(stdout, "left button  b=%d\n",mev->button());
      ZoomMode = ZOOM_NORM;
      ZoomX1   = mev->x();
      ZoomY1   = mev->y();
      ZoomX2   = mev->x();
      ZoomY2   = mev->y();
      zxa=zya=bra=hoa=0;
   } else
   if (mev->button()==MidButton) {   // Point & Click
      ZoomX1   = mev->x();
      ZoomY1   = mev->y();
      ZoomMode = ZOOM_CLICK;
   }
}
void DrawWidget::mouseReleaseEvent (QMouseEvent* mev) 
{
   //fprintf(stdout,"mouseReleaseEvent\n");
   if (ZoomMode == ZOOM_NORM) {
      ZoomX2   = mev->x();
      ZoomY2   = mev->y();
      if ( (mev->state() & ControlButton) != 0) {
         AbortCalc();
         Fractal->ZoomOut(ZoomX1,ZoomY1, ZoomX2,ZoomY2);
         update();
      } else
      if (Fractal->ZoomIn(ZoomX1,ZoomY1, ZoomX2,ZoomY2)) {
         //Berechnung=FALSE;
         //ImageReady=FALSE;
         AbortCalc();
         update();
      }
      ZoomMode = ZOOM_NONE;
   } else
   if (ZoomMode == ZOOM_CLICK) {
      AbortCalc();
      if ( (mev->state() & ControlButton) != 0)
         Fractal->ZoomOut(ZoomX1, ZoomY1, ZoomFactor);
      else
         Fractal->ZoomIn(ZoomX1, ZoomY1, ZoomFactor);
      update();
      ZoomMode = ZOOM_NONE;
   }
}
//  void DrawWidget::mouseDoubleClickEvent (QMouseEvent* mev) 

void DrawWidget::mouseMoveEvent (QMouseEvent* mev) 
{
int zx,zy,br,ho;
int p_active;          // Anzeige: Painter aktiv

   if (ZoomMode == ZOOM_NORM) {
      //fprintf(stdout,"mouseMoveEvent\n");
      ZoomX2   = mev->x();
      ZoomY2   = mev->y();
      zx = QMIN(ZoomX1,ZoomX2);  // Ecke links oben
      zy = QMIN(ZoomY1,ZoomY2);
      br = abs(ZoomX1-ZoomX2);   // Gre
      ho = abs(ZoomY1-ZoomY2);
      p_active = Paint->isActive();
      if (!p_active)
         Paint->begin(this);
      Paint->drawPixmap(zxa,zya,*Pix, zxa,zya, bra,hoa);
      zxa=zx; zya=zy; bra=br; hoa=ho;
      Paint->setPen(white);
      Paint->drawRect(zx,zy, br,ho);
      if (!p_active)
         Paint->end();
   } 
} 

void DrawWidget::CalcFractal(void)
{
int fc;

//fprintf(stdout,"Berechnung\n");
   if (StartUp) return;
   PixCount = 0;
   if (UpdateBool) {
      switch (Fractal->GetFracClass()) {
         case FC_FIELD : PixMax = PicSizeX*PicSizeY;break;
         case FC_ORBIT : PixMax = (PicSizeX+PicSizeY/10)*Fractal->MaxIter;break;
         default: ModVal = 1;
      }
      ModVal = PixMax/33;   //3%-Schritte
   } else {
      ModVal = 1000000000;  // keine Anzeige
      PixMax = 1;
   }
   kmand->SetStatus("", SZ_PROGRESS);

   fc = Fractal->GetFracClass();
   switch (fc) {
      case FC_FIELD : CalcMandel(); break;
      case FC_ORBIT : CalcOrbit(); break;
   }
   kmand->SetStatus("fertig", SZ_PROGRESS);
}

void DrawWidget::PutPixel(int x, int y, long Iter)
{
int b,x1;
char Text[10];

   PixCount++;   // einen Pixel mehr
   if (ColorDeep==8)
      *(Image->scanLine(y)+x) = Iter%256;
   else
   if (ColorDeep==1) {
      b = x%8;
      x1= x/8;
      Iter = Iter%NumColors;
      if (Iter)
         *(Image->scanLine(y)+x1) |= (128>>b);
      else
         *(Image->scanLine(y)+x1) &= ~(128>>b);
   } else {  // 32 Bit
         x1 = x<<2;
      switch (HiColorMode) {
         case 0 :  *(Image->scanLine(y)+x1)   = Iter & 255;   // Rot
                   *(Image->scanLine(y)+x1+1) = 0; //255-(Iter >> 8)&255; // grn
                   *(Image->scanLine(y)+x1+2) = 255-((Iter >> 16)&255);  // Blau
                   *(Image->scanLine(y)+x1+3) = 0; break;

         case 1 :  *(Image->scanLine(y)+x1)   = (char)(sin(Iter*0.05)*127+127);   // Rot
                   *(Image->scanLine(y)+x1+1) = (char)(-sin(Iter*0.03)*127+127); // grn
                   *(Image->scanLine(y)+x1+2) = (char)(sin(Iter*0.08)*127+127);  // Blau
                   *(Image->scanLine(y)+x1+3) = 0; break;
      }         
   }

   if ((PixCount % ModVal)  == 0) {  // Status-Update nach xxx Pixeln
      sprintf(Text, "%3ld%%", (PixCount*100)/PixMax);
      kmand->SetStatus(Text, SZ_PROGRESS);
   }
}

long DrawWidget::GetPixel(int x, int y)
{
int b,x1;

   if (ColorDeep==8)
      return *(Image->scanLine(y)+x);
   else if (ColorDeep==1) {
      b = x%8;
      x1= x/8;
      return (((*(Image->scanLine(y)+x1)) & (128>>b)) != 0);
   } else{
      x1 = x<<2;
      unsigned long b1,b2,b3,b4;
      b1 = *(Image->scanLine(y)+x1);
      b2 = *(Image->scanLine(y)+x1+1);
      b3 = *(Image->scanLine(y)+x1+2);
      b4 = *(Image->scanLine(y)+x1+3);
      return b1 + (b2*256) + (b3*65536);// + (b3<<24);
   }
}

int DrawWidget::TestRect(int x1, int x2, int y1, int y2) {
// Prft, ob alle Punkte auf dem Rechteck(x1y1, x2y2) die gleiche Farbe
// haben und gibt dann "true" (=1) zurck, sonst "False" (=0)
// Die Optimierung geht dahin, mgligst frh zu dem Ergebnis "False"
// zu kommen, da dieser Fall wohl am meisten eintritt
int Ret;
int Lauf;
int Pixel;
// return True;

   // die vier Eckpunkte zuerst vergleichen
   Ret = TRUE;
   Pixel = GetPixel(x1,y1);

   if (Pixel != GetPixel(x2,y1) )
      return TRUE;
   if (Pixel != GetPixel(x2,y2) )
      return TRUE;
   if (Pixel != GetPixel(x1,y2) )
      return TRUE;

   // waagerecht oben
   Lauf = x1+1;
   while ( Ret && (Lauf < x2) )
   {
      Ret = (GetPixel(Lauf,y1) == Pixel) &
        (GetPixel(Lauf,y2) == Pixel);
      Lauf++;
   }

   // senkrecht rechts
   Lauf = y1+1;
   while ( Ret && (Lauf < y2) )
   {
      Ret = (GetPixel(x1, Lauf) == Pixel) &
        (GetPixel(x2, Lauf) == Pixel);
      Lauf++;
   }
   // Alle Punkte auf dem Rahmen haben gleiche Farbe,
   // weitere Berechnung nicht ntig
   return !Ret;
}

void DrawWidget::FillRect(int x1, int x2, int y1, int y2, int Level) {
// Recheck ausmalen
int i,j,X;
unsigned long Color;
   Color = GetPixel(x1,y1);
   x1++;y1++;
   for (i=x1;i<x2;i++)
   for (j=y1;j<y2;j++)
   if (ColorDeep == 32) {
      X = i<<2;
      *(Image->scanLine(j)+X)   = (char)(Color % 256);   // Rot
      *(Image->scanLine(j)+X+1) = (char)((Color >> 8) % 256); // grn
      *(Image->scanLine(j)+X+2) = (char)((Color >> 16) % 256);  // Blau
      *(Image->scanLine(j)+X+3) = 0;
   } else
      PutPixel(i,j,Color);

   if (Level<UpdateLevel)  {
      Pix->convertFromImage(*Image);
//      Paint->drawPixmap(0, 0, *Pix);
      Paint->drawPixmap(x1-1,y1-1,*Pix, x1-1,y1-1, x2-x1+2,y2-y1+2);
   }
   if (Level<(UpdateLevel<<1))
      qApp->processEvents();
}

// Waagerecht teilen
void DrawWidget::Calc2w (int x1, int x2, int y1, int y2, int Level) {
int i, Mitte, Lauf;

   if (!Berechnung) return;
   Mitte = (y2-y1)/2;
   Mitte = y1+Mitte;

   Lauf = 1;
   for (i=x1+1; i<x2; i++) {
      Lauf++;
      PutPixel(i, Mitte, Fractal->CalcPoint(i, Mitte));
      if (!Berechnung) return;
   }
   if (Mitte-1>y1)
   if (TestRect(x1,x2,y1,Mitte))
      Calc2s(x1,x2,y1,Mitte, Level+1);  // obere Seite
   else
      FillRect(x1,x2,y1,Mitte,Level);
   if (!Berechnung) return;

   if (y2>Mitte+1)
   if (TestRect(x1,x2,Mitte,y2))
      Calc2s(x1,x2,Mitte,y2, Level+1);  // untere Seite
   else
      FillRect(x1,x2,Mitte,y2,Level);

   if (Level==UpdateLevel)  {
      Pix->convertFromImage(*Image);
//      Paint->drawPixmap(0, 0, *Pix);
      Paint->drawPixmap(x1,y1,*Pix, x1,y1, x2-x1,y2-y1);
   }
   if (Level==(UpdateLevel<<1))
      qApp->processEvents();
}

// senkrecht teilen
void DrawWidget::Calc2s (int x1, int x2, int y1, int y2, int Level) {
int i, Mitte, Lauf;
 
   if (!Berechnung) return;
   Mitte = (x2-x1)/2;
   Mitte = x1+Mitte;

   Lauf = 1;
   for (i=y1+1; i<y2; i++) {
      Lauf++;
      PutPixel(Mitte, i, Fractal->CalcPoint(Mitte, i));
      if (!Berechnung) return;
   }
   if (Mitte-1>x1)
   if (TestRect(x1,Mitte,y1,y2))
      Calc2w(x1,Mitte,y1,y2,Level+1);  // linke Seite
   else
      FillRect(x1,Mitte,y1,y2,Level);
   if (!Berechnung) return;

   if (x2>Mitte+1)
   if (TestRect(Mitte,x2,y1,y2))
      Calc2w(Mitte,x2,y1,y2,Level+1);  // rechts Seite
   else
      FillRect(Mitte,x2,y1,y2,Level);

   if (Level==UpdateLevel)  {
      Pix->convertFromImage(*Image);
//      Paint->drawPixmap(0, 0, *Pix);
      Paint->drawPixmap(x1,y1,*Pix, x1,y1, x2-x1,y2-y1);
   }
   if (Level==(UpdateLevel<<1))
      qApp->processEvents();
}

void DrawWidget::Calc2(void) {
int i; 

   if (!UpdateBool) UpdateLevel=-1; else  // kein Update
   if (UL<50)  UpdateLevel = 4; else      // feines Update
   if (UL<100) UpdateLevel = 3; else      // mittleres Update
   if (UL<250) UpdateLevel = 2; else      // grobes Update
   UpdateLevel = 1;

   // Rand berechnen
   for (i=0; i< PicSizeX-1; i++) {
      PutPixel(i, 0,       Fractal->CalcPoint(i, 0));
      PutPixel(i, PicSizeY-1, Fractal->CalcPoint(i, PicSizeY-1));
   }

   for (i=1; i< PicSizeY-1; i++) {
      PutPixel(0, i,       Fractal->CalcPoint(0, i));
      PutPixel(PicSizeX-1, i, Fractal->CalcPoint(PicSizeX-1, i));
   }

   Calc2s(0,PicSizeX-1,0,PicSizeY-1,0);

   if (!Berechnung) {     // Wurde die Berechnung von auen abgebrochen
                          // jetzt sofort beenden
      Paint->end();
      ImageReady=FALSE;
      update();
      //fprintf(stdout," ABORT\n");
      //return;
   }
}


void DrawWidget::CalcMandel(void)
{
int x,y;

//fprintf(stdout,"CalcMandel %d  %d\n",Pix->width(),Pix->height());
if (Berechnung) {
   //fprintf(stdout,"CalcMandel rejected\n");
   return;
}
   if(!Paint->isActive()) {   // berschneiden sich zwei Paint-Aufrufe,
                              // den nachfolgenden ignorieren
  //fprintf(stdout,"CalcMandel inside\n");
      //fprintf(stdout, "Repaint: Image calculate\n");

      Paint->begin(this);
      Berechnung = TRUE;
      ImageReady=FALSE;
      //Pix=new QPixmap();

      Fractal->InitField();            // Field-Type init
      if (CalcMethod == CM_RECT) {
         Calc2();
         if (!Berechnung) {
            return;
         }
      } else
      {
         for(y=0;y<PicSizeY;y++) {
            for(x=0;x<PicSizeX;x++) {
               if (!Berechnung) {     // Wurde die Berechnung von auen abgebrochen
                                      // jetzt sofort beenden
                  Paint->end();
                  ImageReady=FALSE;
                  update();
                  //fprintf(stdout," ABORT\n");
                  return;
               }
               PutPixel(x,y, Fractal->CalcPoint(x,y));
            }
            if (UpdateBool)
            if (y%UL == 0) {
               Pix->convertFromImage(*Image);
               Paint->drawPixmap(0,y/UL*UL-UL,*Pix, 0,y/UL*UL-UL, PicSizeX,UL);
            }

            qApp->processEvents();
            //if (Resized) fprintf(stdout," ****** Resize in Berechnung\n");
         }
      }
      if (Berechnung) {
         Pix->convertFromImage(*Image);
         Paint->drawPixmap(0, 0, *Pix);
      }
      Paint->end();   

      //delete Pix;
   }  //else
      //fprintf(stdout,"Painter is active\n");
   Berechnung=FALSE;
   ImageReady = TRUE;
   //fprintf(stdout,"ImageReady=True\n");
}

void DrawWidget::CalcOrbit(void)
{
int Loops;  // Anzahl der Durchlufe
int Lauf;   // aktueller Durchlauf
int ix,iy;  //  Bildpunkt
int Color;  // Farbe fr den Punkt
double li=Fractal->li,re=Fractal->re,un=Fractal->un,ob=Fractal->ob;

   if(!Paint->isActive()) {   // berschneiden sich zwei Paint-Aufrufe,
                              // den nachfolgenden ignorieren

      //fprintf(stdout,"CalcOrbit\n");
      Fractal->OrbX = 0;
      Fractal->OrbY = 0;
      Fractal->OrbZ = 0;
      Fractal->InitOrbit();
      Loops = (PicSizeX+PicSizeY/10)*Fractal->MaxIter;

      Berechnung = TRUE;
      ImageReady = FALSE;
      Color = rand();;
      Paint->begin(this);
 
      for (Lauf=0; Lauf<Loops; Lauf++) {
         Fractal->CalcOrbit();

         ix = (int)((Fractal->OrbX-li)/(re-li)*PicSizeX);
         iy = (int)((Fractal->OrbY-un)/(ob-un)*PicSizeY);

         // Punkt setzen
         if (ix >= 0)
         if (ix < PicSizeX)
         if (iy >=0)
         if (iy < PicSizeY) {
/*            if (ColorDeep==1) {
               b = ix%8;
               x1= ix/8;
               *(Image->scanLine(iy)+x1) |= (1<<b);
            } else
            if (ColorDeep==8) {
               *(Image->scanLine(iy)+ix) = Color % 256;
            } else
            {  // 32 bpp
               x1 = ix<<2;
               *(Image->scanLine(iy)+x1)   = Color & 255;   // Rot
               *(Image->scanLine(iy)+x1+1) = (Color >> 8) &255; // grn
               *(Image->scanLine(iy)+x1+2) = (Color>>16) & 255;  // Blau
               *(Image->scanLine(iy)+x1+3) = 0;
            }
*/
            PutPixel(ix, iy, Color);
         }

         if (Lauf%PicSizeY==0)   // Farbwechsel
            Color = rand();

         if (UpdateBool)
         if (Lauf%(UL*Fractal->MaxIter) == 0) {
            Pix->convertFromImage(*Image);
            Paint->drawPixmap(0,0,*Pix);
         }
         qApp->processEvents();

         if (!Berechnung) {     // Wurde die Berechnung von auen abgebrochen
                                // jetzt sofort beenden
            Paint->end();
            ImageReady=FALSE;
            update();
            //fprintf(stdout," ABORT\n");
            return;
         }
      }//for()

      Pix->convertFromImage(*Image);
      Paint->drawPixmap(0, 0, *Pix);
      Paint->end();
      Berechnung = FALSE;
      ImageReady = TRUE;
      //fprintf(stdout,"Calcorbit: ende\n");
   }
}

void DrawWidget::PaletteInterpolate(int Start, int Ende)
//  Farbverlauf zwischen den Farbpaletteneintrag Start und Ende erzeugen
{
int Lauf;
int diff;
int sr,sg,sb,er,eg,eb;  // Farbwerte am Anfang und Ende
int r,g,b;

  diff = (Ende-Start);   // Anzahl der zu ndernden Werte
  if (diff <= 0) return;   // End < Anf
  // Farbanteile am Start
  sr = qRed(ColorMap[Start]);
  sg = qGreen(ColorMap[Start]);
  sb = qBlue(ColorMap[Start]);
  // Farbanteile am Ende
  er = qRed(ColorMap[Ende]);
  eg = qGreen(ColorMap[Ende]);
  eb = qBlue(ColorMap[Ende]);
  // Farben interpolieren
  for (Lauf=Start+1; Lauf<Ende; Lauf++) {
    r = sr + (er-sr)*(Lauf-Start)/diff;
    g = sg + (eg-sg)*(Lauf-Start)/diff;
    b = sb + (eb-sb)*(Lauf-Start)/diff;
    //Image->setColor(Lauf, qRgb(r,g,b));
    ColorMap[Lauf] = qRgb(r,g,b);
  }
}

void DrawWidget::CreatePalette(void)
{
  if (ColorDeep==1) {
     ColorMap[0] = 0;  // Schwarz
     ColorMap[1] = qRgb(255,255,255);  // Weiss
  } else 
  if (ColorDeep==8) {
      //fprintf(stdout, "KMandel::InitPalette()\n");
     ColorMap[0] = qRgb(0,0,0);//   = 0x00000000;     // schwarz
     ColorMap[31] = qRgb(0,0,0xFF); //  = 0x000000FF;     // blau
     PaletteInterpolate(0,31);
     ColorMap[63] = qRgb(0x80, 0x80, 0x80); //  = 0x00800080;     // rot-blau (dunkel)
     PaletteInterpolate(31,63);
     ColorMap[95] = qRgb(0xFF,0,0);//  = 0x00FF0000;     // rot
     PaletteInterpolate(63, 95);
     ColorMap[127] =qRgb(0x80,0x80,0);// = 0x00808000;     // rot-grn (dunkel)
     PaletteInterpolate(95,127);
     ColorMap[159]= qRgb(0,0xB0,0);// = 0x0000FF00;     // grn
     PaletteInterpolate(127,159);
     ColorMap[191]=qRgb(0,0x80,0x80);// = 0x00008080;     // blau-grn (dunkel)
     PaletteInterpolate(159,191);
     ColorMap[223]=qRgb(0x80,0x80,0x80);// = 0x00808080;     // grau
     PaletteInterpolate(191, 223);
     ColorMap[255]=0;// = 0x00000000;     // schwarz
     PaletteInterpolate(223,255);
   } // else no Palette
}

void DrawWidget::InitPalette(void)
// Farbpalette in das Image-Objekt bertragen
{
unsigned int lauf;
   
   if (ColorDeep > 8) return;  // Truecolor ohne Farbpalette

   for (lauf=0; lauf<NumColors; lauf++)
      Image->setColor(lauf, ColorMap[lauf]);
}

void DrawWidget::NewFractal(int FracType) {
//fprintf(stdout," Fractype = %d\n",FracType);
   AbortCalc();
   Image->fill(0);
   delete Fractal;
//fprintf(stdout,"nach delete Fractal\n");
   switch (FracType) {
      case FT_MANDEL: Fractal = new MandelBrot();
                      break;
      case FT_JULIA:  Fractal = new Julia();
                      break;
      case FT_MLAMBDA:  Fractal = new LMandel();
                      break;
      case FT_JLAMBDA: Fractal = new LJulia();
                      break;
      case FT_INVMANDEL: Fractal = new InvMand();
                      break;
      case FT_GINGER: Fractal = new Ginger();
                      break;
      case FT_GALAXY: Fractal = new Galaxy();
                      break;
      case FT_HOP:    Fractal = new Hop();
                      break;
      case FT_CHIP:   Fractal = new Chip();
                      break;
      case FT_MANDEL_N:   Fractal = new MandelHochN();
                      break;
      case FT_JULIA_N:   Fractal = new JuliaHochN();
                      break;
      case FT_TRIPLE: Fractal = new Triple();
                      break;
      case FT_HENON:  Fractal = new Henon();
                      break;
      case FT_BARNSLEYJA:  Fractal = new BarnsleyJA();
                      break;
      case FT_BARNSLEYB:  Fractal = new BarnsleyB();
                      break;
      case FT_BARNSLEYC:  Fractal = new BarnsleyC();
                      break;
      case FT_ORBIT3:  Fractal = new Orbit3();
                      break;
      case FT_WANKEL:  Fractal = new Wankel();
                      break;
      case FT_QUADRAT:  Fractal = new Quadrat();
                      break;
      case FT_LIAPI:  Fractal = new Liapi();
                      break;
      case FT_STAR:   Fractal = new Star();
                      break;
      case FT_LORENZ: Fractal = new Lorenz();
                      break;
      case FT_GENETIC: Fractal = new Genetic();
                      break;
      case FT_PHEONIX: Fractal = new Pheonix();
                      break;
      case FT_POCKEN: Fractal = new Pocken();
                      break;
      default:        Fractal = new BaseFract();
   }
   Fractal->Init(PicSizeX, PicSizeY);
//fprintf(stdout,"nach fractal->init\n");
   if (kmand)
      kmand->SetStatus(Fractal->GetFracName(), SZ_TYPE);
//fprintf(stdout,"nach setstatus\n");
   update();
//fprintf(stdout,"nach update\n");
}

int DrawWidget::LoadColorMap(char* MapName)
{
FILE* f;
int r,g,b,Lauf;
char Line[128];

   f=fopen(MapName,"rt");
   if (f==NULL) return FALSE;
   Lauf=0;
   while (!feof(f)) {
      fgets(Line, 128, f);
      sscanf(Line,"%d %d %d",&r,&g,&b);
      //fprintf(stdout,"r=%d g=%d b=%d\n",r,g,b);
      ColorMap[Lauf++] = qRgb(r,g,b);
      if (Lauf >= 256) break;  // mehr als 256 Farben nicht mglich
   }

   fclose(f);
   InitPalette();
   update();
   return TRUE;
}

int DrawWidget::SelectColorMap()
{
   if (ColorDeep>8) {
      QMessageBox::information(this,"No Colormap",
                      "No Colormap in Hi- or Truecolor Mode.",
                      QMessageBox::Ok);
      return FALSE;
   } else {

      QString f = QFileDialog::getOpenFileName( 0, "*.map", this );
      if ( !f.isEmpty() ) {
         // the user selected a valid existing file
         return LoadColorMap(f.data());
      } else {
         // the user cancelled the dialog
         return FALSE;
      }
   }
}

void DrawWidget::ChangePicParams(int sx, int sy, int cdeep)
{// Bildparameter ndern

   AbortCalc();
//fprintf(stdout,"changpicparams: abort\n");
   if (ColorDeep!=cdeep) {
//fprintf(stdout,"Colordeepchange\n");
      switch(cdeep) {
         case 1 : ColorDeep = 1;
                  NumColors = 2;
                  break;
         case 8 : ColorDeep = 8;
                  NumColors = 256;
                  break;
         case 32: ColorDeep = 32;
                  NumColors = 0;
                  break;
         default: ColorDeep=8;
                  NumColors=256;
      }
      CreatePalette();
   }

   SetPicSize(sx,sy);  // neue Gre
   delete Image;
   Image = new QImage(PicSizeX, PicSizeY, ColorDeep, NumColors,QImage::BigEndian);
   InitPalette();
   update();
}

int DrawWidget::SaveColorMap(void)
{           // Farbpalette speichern
FILE* f;
int Lauf;

QString DateiName = QFileDialog::getSaveFileName( 0, "*.map", this );
   if ( !DateiName.isEmpty() ) {     // the user gave a file name
      if (DateiName.right(4) != ".map")     // Endung anfgen, falls
         DateiName += ".map";               // keine angegeben wurde
      if (QFile::exists(DateiName.data())){ // Datei existiert bereits
         if (!QMessageBox::query("Confirm","Overwrite the existing file?"))
            return TRUE;
      }

      f = fopen(DateiName.data(),"wt");
      if (f==0) return FALSE;         // Fehler beim ffnen

      for (Lauf=0; Lauf <256; Lauf++) {
         fprintf(f, "%d %d %d\n", qRed(ColorMap[Lauf]), qGreen(ColorMap[Lauf]), qBlue(ColorMap[Lauf]));
      }
      fclose(f);
   }
   return TRUE;
}

void DrawWidget::EditColorMap(void)
{
PalEditDlg PalEdit(&(ColorMap[0]), NumColors, 0, "Palette editieren");
int Lauf;

   if (ColorDeep>8) {
      QMessageBox::information(this,"No Colormap",
                      "No Colormap in Hi- or Truecolor Mode.",
                      QMessageBox::Ok);
   } else
   if (PalEdit.exec()) {
      for (Lauf=0; Lauf<256; Lauf++) {
         ColorMap[Lauf] = PalEdit.GetColor(Lauf);
         InitPalette();
         update();
      }
   }
}

void DrawWidget::MenuSavePic()
{ // Image speichern
   QString f = QFileDialog::getSaveFileName( 0, "*.bmp", this );
   if ( !f.isEmpty() ) {
      if (f.right(4) != ".bmp")     // Endung anfgen, falls
         f += ".bmp";               // keine angegeben wurde
      if (QFile::exists(f.data())){ // Datei existiert bereits
         if (!QMessageBox::query("Confirm","Overwrite the existing file?"))
            return;
      }

      if (!Image->save(f.data(),"BMP"))
         QMessageBox::message("Fehler","Error saving Picture.");
   }
}

void DrawWidget::LoadPic(char* DateiName)
// Gespeichertes Bild laden
{
   Image->load(DateiName, "BMP");
   SetPicSize(Image->width(),Image->height());
   ColorDeep = Image->depth();
   ImageReady = TRUE;
   Berechnung = FALSE;
//fprintf(stdout,"Depth=%d, NumColors=%d\n",ColorDeep,NumColors);
//if (Image->bitOrder() == QImage::BigEndian)
//   fprintf(stdout,"Bitorder ok\n");
//else
//   fprintf(stdout,"Bitorder Fehler\n");
}


void DrawWidget::AbortCalc(void)
// Berechnung abbrechen 
{
//fprintf(stdout,"Abortcalc\n");
   Berechnung=FALSE;
   ImageReady=FALSE; 
   Image->fill(0);
}

void DrawWidget::MenuZoomIn()
{ // Reinzoomen
   AbortCalc();
   Fractal->ZoomIn(PicSizeX>>1,PicSizeY>>1,ZoomFactor);
   update();
}

void DrawWidget::MenuZoomOut()
{ // Rauszoomen
   AbortCalc();
   Fractal->ZoomOut(PicSizeX>>1,PicSizeY>>1,ZoomFactor);
   update();
}

void DrawWidget::GetParamsBlock(struct MandDataType* MandData)
// Daten setzen, die in der Parameterdatei gespeichert werden sollen
{
   MandData->ZoomFactor = ZoomFactor;
   MandData->ZoomFactorAnimation = ZoomFactorAnimation;
   MandData->ZoomAnimationPics = ZoomAnimationPics;
   MandData->UpdateInterval = UpdateInterval;
   MandData->UpdateBool = UpdateBool;
   MandData->CalcMode = CalcMethod;
}

void DrawWidget::SetParamsBlock(struct MandDataType* MandData)
// Daten setzen, die aus einer Parameterdatei gelesen wurden
{
   ZoomFactor = MandData->ZoomFactor;
   ZoomFactorAnimation = MandData->ZoomFactorAnimation;
   ZoomAnimationPics = MandData->ZoomAnimationPics;
   UpdateInterval = MandData->UpdateInterval;
   UpdateBool = MandData->UpdateBool;
   CalcMethod = MandData->CalcMode;
}

void DrawWidget::Fertig()
{
   fprintf(stdout,"FERTIG %s",(ImageReady?"ok\n":"Fehler\n"));
}

void DrawWidget::PicToColorMap(void) {
// Farbpalette aus dem gelesenem Bild benutzen
unsigned int lauf;

   if (ColorDeep > 8) return;  // Truecolor ohne Farbpalette

   for (lauf=0; lauf<NumColors; lauf++)
      ColorMap[lauf] = Image->color(lauf);
}

KMandel::KMandel(const char *name )
        : KTopLevelWidget( name )
{
QString Datei;

    //Btn = new QPushButton("Hallo",this,"Btn");
    //Btn->resize(250,200);

    Client = new DrawWidget(this);
    setView(Client);
    //Client->resize(150,100);

    //SetClientSize(MAXX, MAXY);
    InitMenue();
    InitStatusZeile();
    InitToolBar();
    // SetKeys();
    Config = KApplication::getKApplication()->getConfig();
    Config->setGroup("Allgemeines");
    Datei = Config->readEntry("StartParameter","");
    if (!Datei.isEmpty())
       LoadParamFile(&Datei);
//fprintf(stdout,"kmandel init ende\n");
}

KMandel::~KMandel()
{
   if (StatusZeile) delete StatusZeile;
   if (ToolBar) delete ToolBar;
   if (MainMenu) delete MainMenu;
}

void KMandel::SetClientSize(int Breite, int Hoehe)
// Gre des Clientbereiches durch ndern der Gre des
// Frames auf die angegebene Gre bringen
{
   int DiffX,DiffY;
   int NewX, NewY;
   

// fprintf(stdout, "SetClientSize: width()=%d height()=%d\n",Client->width(),Client->height());
   DiffX = Client->width()-Breite;
   DiffY = Client->height()-Hoehe;
// fprintf(stdout, "SetClientSize: DiffX=%d DiffY=%d\n",DiffX,DiffY);
   NewX  = width()-DiffX;
   NewY  = height()-DiffY;
// fprintf(stdout, "SetClientSize: width=%d height=%d\n",width(),height());
// fprintf(stdout, "SetClientSize: NewX=%d NewY=%d\n",NewX,NewY);
   resize(NewX, NewY);

/*   if (Client->height() != Hoehe) {
      // "nachtriggern"
      fprintf(stdout,"SetClientSize: Hhe nicht erreicht\n");
      DiffY = Client->height()-Hoehe;
      NewY  = height()-DiffY;
      resize(NewX, NewY);
      if (Client->height() != Hoehe)
         fprintf(stdout,"Hhe wieder nicht erreicht, forget it\n");
   }*/
//   qApp->processEvents();
// fprintf(stdout, "SetClientSize: Ende\n");
}

void KMandel::InitMenue()
// Initialisierung des Hauptmens
{
    //fprintf(stdout,"KMandel::InitMenue\n");

    // Datei-Men
    QPopupMenu* FileMenu = new QPopupMenu; 
    FileMenu->insertItem(TRS("&New"), this, SLOT(MenuNew()),CTRL+Key_N);
    FileMenu->insertItem(TRS("&Open"), this, SLOT(MenuOpen()), CTRL+Key_O );
    FileMenu->insertItem(TRS("&Save Pic"),Client,SLOT(MenuSavePic()) );
    FileMenu->insertItem(TRS("Save &Params"), this, SLOT(MenuWriteParams()), CTRL+Key_S );
    FileMenu->insertItem(TRS("&Filesettings"), this, SLOT(MenuFileSettings()) );
    FileMenu->insertSeparator();
    FileMenu->insertItem(TRS("&Exit"), qApp, SLOT(quit()), ALT+Key_X );

    // Fraktal-Men
    QPopupMenu* FraktalMenu = new QPopupMenu;
    FraktalMenu->insertItem(TRS("&Type"), this, SLOT(ShowFracTypeDlg()),Key_F5 );
    FraktalMenu->insertItem(TRS("&Parameter"), this, SLOT(ShowFracParamsDlg()), Key_F6 );
    FraktalMenu->insertItem(TRS("Type-&specific"), this , SLOT(MenuSpecialParams()), Key_F7 );

    // Farbpalette
    QPopupMenu* PicMenu = new QPopupMenu;
    PicMenu->insertItem(TRS("&General Settings"), this, SLOT(MenuPicSettings()), Key_F9 );
    PicMenu->insertItem(TRS("&Edit Palette"), this, SLOT(MenuEditPal()),Key_F10 );
    PicMenu->insertItem(TRS("&Load Palette"), this, SLOT(MenuLoadColorMap()),Key_F11 );
    PicMenu->insertItem(TRS("&Save Palette"), this, SLOT(MenuSaveColorMap()),Key_F12 );

    // Zooming
    QPopupMenu* ZoomMenu = new QPopupMenu;
    ZoomMenu->insertItem(TRS("Zoom &in"), Client, SLOT(MenuZoomIn()), Key_Plus );
    ZoomMenu->insertItem(TRS("Zoom &out"), Client, SLOT(MenuZoomOut()), Key_Minus );
    ZoomMenu->insertItem(TRS("Zoom &Setup"), this, SLOT(MenuZoomSettings()), Key_Asterisk );

    // ber-Dialoch und Hilfe
    QPopupMenu* MUMenu = new QPopupMenu;
    MUMenu->insertItem(klocale->translate("&About"), this, SLOT(ShowAboutDlg()) );
    MUMenu->insertSeparator();
    MUMenu->insertItem(TRS("&Hilfe"), this, SLOT(MenuHelp()), Key_F1 );

    MainMenu = new KMenuBar(this);
    MainMenu->insertItem(TRS("&File"), FileMenu, MID_FILE);
    MainMenu->insertItem(TRS("Fra&ctal"), FraktalMenu);
    MainMenu->insertItem(TRS("&Picture"), PicMenu);
    MainMenu->insertItem(TRS("&Zoom"), ZoomMenu);
    MainMenu->insertSeparator();
    MainMenu->insertItem(TRS("&Help"), MUMenu);
    MainMenu->show();
    setMenu(MainMenu);
}

void KMandel::InitStatusZeile()
{
   StatusZeile = new KStatusBar(this, "StatusZeile");
   StatusZeile->insertItem("XXXXXXXXXXXXXXX", SZ_TYPE);
   StatusZeile->changeItem("Mandelbrot", SZ_TYPE);
   StatusZeile->insertItem("fertig", SZ_PROGRESS);
   StatusZeile->changeItem("", SZ_PROGRESS);
   StatusZeile->insertItem("", 4711);
   setStatusBar(StatusZeile);
}

void KMandel::InitToolBar(void)
{
   ToolBar = new KToolBar(this, "ToolBar");
   
#define I(f,o,s,h) ToolBar->insertButton( kapp->getIconLoader()->loadIcon(f),\
	 0, SIGNAL(clicked()), o, SLOT(s()), TRUE, h);

   // File
   I( "filenew2.xpm",     this,     MenuNew,         "Reinit this Type")
   I( "fileopen.xpm",     this,     MenuOpen,        "Open")
   I( "filefloppy.xpm",   this,     MenuWriteParams, "Save Params")
   ToolBar->insertSeparator();ToolBar->insertSeparator();
   // Fractal
   I( "filenew.xpm",      this,     ShowFracTypeDlg, "New Fractal")
   I( "openbook.xpm",     this,     ShowFracParamsDlg, "Generic Parameters")
   I( "exclamation.xpm",  this,     MenuSpecialParams, "Special Parameters")
   ToolBar->insertSeparator();ToolBar->insertSeparator();
   //Zoom
   I( "viewmag+.xpm",     Client,   MenuZoomIn,      "Zoom in")
   I( "viewmag-.xpm",     Client,   MenuZoomOut,     "Zoom out")
   ToolBar->insertSeparator();ToolBar->insertSeparator();
   // Palette
   I( "largetext.xpm",    this,     MenuEditPal,     "Edit Palette")
   I( "fileopen.xpm",     this,     MenuLoadColorMap,"Load Palette")
   I( "filefloppy.xpm",   this,     MenuSaveColorMap,"Save Palette")
   ToolBar->insertSeparator();ToolBar->insertSeparator();
   I( "help.xpm",         this,     MenuHelp,        "Help!")

   addToolBar(ToolBar);
}

void KMandel::ShowAboutDlg() {
   //fprintf(stdout, "ShowAboutDlg\n");
   AboutDlg About(0,"AboutDlg");
   About.exec();
}

void KMandel::ShowFracTypeDlg() {
   FracTypeDlg FracType(0,"FracTypeDlg");
   FracType.SetFracType(Client->Fractal->GetFracType());
   if (FracType.exec()) {
      //fprintf(stdout,"Dialog ok\n");
      Client->NewFractal(FracType.GetFracType());
   } //else
      //fprintf(stdout,"Dialog cancel\n");
}

void KMandel::ShowFracParamsDlg()
{
char* Strg;
   FracParamsDlg FracParams(  Client, 0, "Fraktalparameter");
//   FracParams.SetCornerParams(Client->Fractal->li,
//                              Client->Fractal->un,
//                              Client->Fractal->re,
//                              Client->Fractal->ob);
//   FracParams.SetGeneralParams(Client->Fractal->MaxIter);
//   FracParams.SetConvParams(  Client->Fractal->ConvMode,
//                              Client->Fractal->BailOut);
//   FracParams.SetParamParams( &Client->Fractal->p[0]);
//   FracParams.SetInfoParams(  Client->Fractal->GetFracType(),
//                              Client->Fractal->GetFracName(),
//			      Client->Fractal->GetFracInfo());
   if (FracParams.exec()) {
      // fprintf(stdout,"Dialog ok\n");
      Client->AbortCalc();
      //if (!
      Strg=FracParams.SetParams();
      if (Strg) fprintf(stdout,"Parameter-Fehler : %s\n", Strg);
//      FracParams.GetCornerParams(&Client->Fractal->li,
//                                 &Client->Fractal->un,
//                                 &Client->Fractal->re,
//                                 &Client->Fractal->ob); //)
//       //fprintf(stdout," Param-Error\n");
//      FracParams.GetGeneralParams(&Client->Fractal->MaxIter);
//      FracParams.GetConvParams(  &Client->Fractal->ConvMode,
//                                 &Client->Fractal->BailOut);
//      FracParams.GetParamParams( &Client->Fractal->p[0]);
      Client->Fractal->ReInit();
      Client->update();
   } //else
     // fprintf(stdout,"Dialog cancel\n");
}

void KMandel::MenuNew()
{
   Client->NewFractal(Client->Fractal->GetFracType());  // reinit
}

void KMandel::MenuLoadColorMap()
{
   Client->SelectColorMap();  // reinit
}

void KMandel::MenuWriteParams()
{
QString DateiName;
ParameterFile* Par;
struct MandDataType MandData;
int Res,S;
char Info[] = "Fractal created by Mandel Ultimativ V 0.35 (c) CFJH";

   DateiName = QFileDialog::getSaveFileName( 0, "*.par", 0, "Parameter speichern");
   if (DateiName.isEmpty()) return;  // Dialog Cancel

   if (DateiName.right(4) != ".par") {
      DateiName += ".par";
      //fprintf(stdout,"keine Endung '.par'\n");
      //fprintf(stdout,"Endung jetzt >%s<\n",DN.data());
   }

   if (QFile::exists(DateiName.data())){ // Datei existiert bereits
      if (!QMessageBox::query("Confirm","Overwrite the existing file?"))
         return;
   }

   Client->Fractal->GetParamsBlock(&MandData); // Frak-Parameter speichern
   Client->GetParamsBlock(&MandData);          // Gen-Parameter speichern

   // Parameter mit dem neuen Parameter-Objekt speichern
   Par = new ParameterFile(DateiName);
   Res = Par->Open(PAR_WRITE);
   if (Res == PAR_OK) {
      Par->WriteBlock('M','U', &MandData, sizeof(MandData));
      if (Client->SaveColorMapBool)
         Par->WriteBlock('C','M', &(Client->ColorMap), sizeof(Client->ColorMap));
      S = Client->Fractal->GetSpecialParamsBytes();
//      fprintf(stdout,"Special Params: Size =%d\n",S);
      if (S>0)
         //fprintf(stdout,"ptr=%d\n",Client->Fractal->GetSpecialParamsBlock());
         Par->WriteBlock('T','S', Client->Fractal->GetSpecialParamsBlock(),S);
      Par->WriteBlock('I','N', &Info, sizeof(Info));
      Par->Close();
   }
   delete Par;
   if (Res!=PAR_OK) {
      QMessageBox::message("Fehler","Parameter File Writing Error");
      return;
   }
}

void KMandel::InitParameterRec(struct MandDataType* MD)
{
int Lauf;

   // Version 0.2a
   MD->FractalType = 0;
   MD->li = -3.0;
   MD->re =  3.0;
   MD->un = -2.0;
   MD->ob =  2.0;
   MD->MaxIter = 256;
   MD->BailOut = 4.0;
   MD->ConvMode = 0;
   for (Lauf=0;Lauf<16;Lauf++) {
      MD->p[Lauf] = 0.0;
      MD->l[Lauf] = 0;
   }
   // Version 0.2b
   MD->ZoomFactor = 5.0;
   MD->ZoomFactorAnimation = 1.03;
   MD->ZoomAnimationPics = 5;
   MD->UpdateInterval = 50;
   MD->UpdateBool = TRUE;
   // V0.21
   MD->InsideMode = INS_SOLIDCOLOR;
   MD->OutsideMode = OUT_ITER;
   MD->InsideColor = 0;
   MD->OutsideColor = 128;
   MD->PotentialSlope = 500.0;
   MD->CalcMode = CM_LINE;
}

void KMandel::MenuOpen()
{
QString DN,D2;

   // Dateiname erfragen
   DN = QFileDialog::getOpenFileName(0, "*.par", this);
   if (DN.isEmpty()) return;   // Cancel im Dialoch

   if (DN.right(4) != ".par") {
      DN += ".par";
      //fprintf(stdout,"keine Endung '.par'\n");
      //fprintf(stdout,"Endung jetzt >%s<\n",DN.data());
   }

   LoadParamFile(&DN);
   return;

/*   InitParameterRec(&MandData);

   // Header der Parameterdatei lesen
   Par = new ParameterFile(DN.data());
   Res = Par->Open(PAR_READ);
   if (Res != PAR_OK) {  // Fehler beim ffnen
      delete Par;
      QMessageBox::message("Fehler", "File Open Error");
      return;
   }
   // Blockgre ermitteln
   Size = sizeof(MandData);
   Res = Par->BlockSize('M','U');
   if (Res < 0) {
      QMessageBox::message("Fehler", "No Mandel Ultimativ Parameterfile");
      Par->Close();
      delete Par;
      return;
   }
   if (Res < Size) {  // Weniger Daten, frhere Version
      QMessageBox::message("Notice", "Parameterfile created with older programversion");
   } else
   if (Res > Size) {  // Mehr Daten, neuere Dateiversion
      QMessageBox::message("Notice", "Parameterfile created with newer programversion");
      Res = Size;     // Anzahl der zu lesenden Zeichen begrenzen
   }

   // Fraktalparameter lesen
   Res = Par->ReadBlock('M','U',&MandData, Res);
   //Client->AbortCalc();
   Client->NewFractal(MandData.FractalType);
   Client->Fractal->SetParamsBlock(&MandData);// Fraktal-Parameter setzen
   Client->SetParamsBlock(&MandData);         // Sonstige Parameter setzen

   // Farbpalette lesen, falls vorhanden
   Res = Par->ReadBlock('C','M', &(Client->ColorMap), sizeof(Client->ColorMap));
   if (Res == PAR_OK)
      Client->InitPalette();

   Par->Close();
   delete Par;

   // Bild suchen und laden, falls vorhanden
   DN.remove(DN.length()-4, 4);  // Endung ".par" entfernen
   DN += ".bmp";                 // Endung ".bmp" anfgen
//fprintf(stdout,"picname=%s\n",DN.data());
   if (QFile::exists(DN.data())) {
      // zugehriges Bild gefunden, laden und anzeigen
      Client->LoadPic(DN.data());
      SetClientSize(Client->GetSizeX(), Client->GetSizeY());
   switch (Client->ColorDeep) {
      case  1 : Client->NumColors = 2;  break;
      case  8 : Client->NumColors = 256; break;
      case 32 : Client->NumColors = 0; break;
      default:fprintf(stdout,"Unbekannte Farbtiefe %d!", Client->ColorDeep);
   }
//fprintf(stdout,"CDeep=%d  NumC=%ld\n",Client->ColorDeep,Client->NumColors);

      Client->LoadPic(DN.data());
      Client->PicToColorMap();     // Farbpalette aus dem Bild benutzen
      Client->Berechnung = FALSE;
      Client->ImageReady = TRUE;
//fprintf(stdout,"Image load ende\n");
   }
   //Client->update();
//fprintf(stdout,"MenuOpen ende\n");
fprintf(stdout,"CDeep=%d  NumC=%ld\n",Client->ColorDeep,Client->NumColors);
*/
}

void KMandel::MenuHelp()
{
//   invokeHTMLHelp("","");
//KApplication::getKApplication()->invokeHTMLHelp("","");
//fprintf(stdout,"Help called...\n");
   kapp->invokeHTMLHelp("","");
}

void KMandel::MenuSpecialParams()
{
   if (Client->Fractal->SetupFrac(Client->Fractal))
   Client->AbortCalc();
   Client->Fractal->ReInit();
   Client->update();
}

void KMandel::MenuPicSettings()
{ // generelle Bildeinstellung
PicSetDlg PsDlg(Client, 0, "Picturesettings");

   if (PsDlg.exec())      // Dialog ok
      if(PsDlg.SetValues())
         SetClientSize(Client->GetSizeX(),Client->GetSizeY());
}

void KMandel::MenuSaveColorMap()
{ // Farbpalette speichern
   if (Client->ColorDeep>8) {
      QMessageBox::information(this,"No Colormap",
                      "No Colormap in Hi- or Truecolor Mode.",
                      QMessageBox::Ok);
   } else {
      if (!Client->SaveColorMap())
         QMessageBox::message("Fehler","Error saving Colormap");
   }
}

void KMandel::MenuEditPal()
{ // Farbpalette bearbeiten
   Client->EditColorMap();
}

void KMandel::resizeEvent( QResizeEvent * qre)
{
// fprintf(stdout,"KMandel::resizeEvent\n");
KTopLevelWidget::resizeEvent(qre);
}

void KMandel::MenuZoomSettings()
{ // Zoomeinstellungen
ZoomSettingsDlg ZoomSettings(Client, this, "ZoomSettings");

   if (ZoomSettings.exec()) {
      ZoomSettings.SetValues();
   }
}

void KMandel::MenuFileSettings()
{ // Dateiparameter einstellen
FileSettingsDlg FileSettings(Client, this, "FileSettings");

   if (FileSettings.exec()) {
      FileSettings.SetValues();
   }
}

void KMandel::SetStatus(char* Text, int ID)
{
   StatusZeile->changeItem(Text, ID);
}

void KMandel::SetKeys(void)
{
	kKeys->registerWidget("KMandel", this);

#define AKCF(f,k,o,s)	kKeys->addKey(f, k);\
			kKeys->connectFunction( "KMandel", f, o, SLOT(s()));
   // File
   AKCF(TRS("&New"),          "CTRL+N",       this,   MenuNew       )
   AKCF("Open",         "CTRL+O",       this,   MenuOpen      )
   AKCF("Save Params",  "CTRL+S",       this,   MenuWriteParams)
   AKCF("Quit",	        "ALT+X",        qApp,   quit          )
   // Factal
//   AKCF("FracType",     "F5",           this,   ShowFracTypeDlg)
   AKCF("FracParams",   "F6",           this,   ShowFracParamsDlg)
   AKCF("FracSpecials", "F7",           this,   MenuSpecialParams)
   // Palette
   AKCF("PalGeneral",   "F9",           this,   MenuPicSettings)
   AKCF("PalEdit",      "F10",          this,   MenuEditPal)
   AKCF("PalLoad",      "F11",          this,   MenuLoadColorMap)
   AKCF("PalSave",      "F12",          this,   MenuSaveColorMap)
   // Zoom
   AKCF("Zoom in",      "Plus",	        Client, MenuZoomIn    )
   AKCF("Zoom out",     "Minus",        Client, MenuZoomOut   )
   AKCF("ZoomSettings", "Asterisk",     this,   MenuZoomSettings)
   // Hilfe
   AKCF("Help",         "F1",           this,   MenuHelp      )

#undef AKCF

//	config->setGroup( "KMandel" );
}

int  KMandel::LoadParamFile(QString* DN)
{
int Res, Size;
ParameterFile* Par;
struct MandDataType MandData;

   InitParameterRec(&MandData);

   // Header der Parameterdatei lesen
   Par = new ParameterFile(DN->data());
   Res = Par->Open(PAR_READ);
   if (Res != PAR_OK) {  // Fehler beim ffnen
      delete Par;
      QMessageBox::message("Fehler", "File Open Error");
      return FALSE;
   }
   // Blockgre ermitteln
   Size = sizeof(MandData);
   Res = Par->BlockSize('M','U');
   if (Res < 0) {
      QMessageBox::message("Fehler", "No Mandel Ultimativ Parameterfile");
      Par->Close();
      delete Par;
      return FALSE;
   }
   if (Res < Size) {  // Weniger Daten, frhere Version
      QMessageBox::message("Notice", "Parameterfile created with older programversion");
   } else
   if (Res > Size) {  // Mehr Daten, neuere Dateiversion
      QMessageBox::message("Notice", "Parameterfile created with newer programversion");
      Res = Size;     // Anzahl der zu lesenden Zeichen begrenzen
   }
//fprintf(stdout,"vor Fraktalparameter lesen\n");
   // Fraktalparameter lesen
   Res = Par->ReadBlock('M','U',&MandData, Res);
//fprintf(stdout,"nach ReadBlock\n");
   //Client->AbortCalc();
   Client->NewFractal(MandData.FractalType);
//fprintf(stdout,"nach NewFractal\n");
   Client->Fractal->SetParamsBlock(&MandData);// Fraktal-Parameter setzen
//fprintf(stdout,"nach SetParamsBlock\n");
   Client->SetParamsBlock(&MandData);         // Sonstige Parameter setzen
//fprintf(stdout,"nach Fraktalparameter lesen\n");

   Res = Client->Fractal->GetSpecialParamsBytes();
   if (Res > 0) {
//      fprintf(stdout,"Lese Special-Params, %d\n",Res);
      Par->ReadBlock('T','S', Client->Fractal->GetSpecialParamsBlock(), Res);
      Client->Fractal->PutSpecialParamsBlock(); // SpecParams initialisieren
   }

   // Farbpalette lesen, falls vorhanden
   Res = Par->ReadBlock('C','M', &(Client->ColorMap), sizeof(Client->ColorMap));
   if (Res == PAR_OK)
      Client->InitPalette();

   Par->Close();
   delete Par;

   // Bild suchen und laden, falls vorhanden
   DN->remove(DN->length()-4, 4);  // Endung ".par" entfernen
 //  (*DN) += ".bmp";                 // Endung ".bmp" anfgen
   DN->append(".bmp");
//fprintf(stdout,"picname=%s\n",DN->data());
   if (QFile::exists(DN->data())) {
      // zugehriges Bild gefunden, laden und anzeigen
      Client->LoadPic(DN->data());
      SetClientSize(Client->GetSizeX(), Client->GetSizeY());
   switch (Client->ColorDeep) {
      case  1 : Client->NumColors = 2;  break;
      case  8 : Client->NumColors = 256; break;
      case 32 : Client->NumColors = 0; break;
      default:fprintf(stdout,"Unbekannte Farbtiefe %d!", Client->ColorDeep);
   }
//fprintf(stdout,"CDeep=%d  NumC=%ld\n",Client->ColorDeep,Client->NumColors);

      Client->LoadPic(DN->data());
      Client->PicToColorMap();     // Farbpalette aus dem Bild benutzen
      Client->Berechnung = FALSE;
      Client->ImageReady = TRUE;
//fprintf(stdout,"Image load ende\n");
   }


   //Client->update();
//fprintf(stdout,"MenuOpen ende\n");
//fprintf(stdout,"CDeep=%d  NumC=%ld\n",Client->ColorDeep,Client->NumColors);
   return TRUE;
}

int main( int argc, char **argv )
{
int Res;
   StartUp = TRUE;
fprintf(stdout,"MU-START\n");
   kmand = NULL;
//    KApplication MandelApp = KApplication( argc, argv );
//    MandelApp MApp = MandelApp(argc, argv, "Mandel Ultimativ");
    MApp = new MandelApp(argc, argv, "kmandel");
    
    kmand = new KMandel("KMandel");  // Hauptfenster
    // if (kmand==NULL)  fprintf(stdout,"kmand=NULL\n");

    MApp->setMainWidget( kmand );
//fprintf(stdout,"vor resize\n");
    kmand->resize(500,449);
//fprintf(stdout,"nach resize\n");
//fprintf(stdout, "nach kmand->resize\n");
    kmand->show();
    kmand->setCaption("Mandel Ultimativ");
    StartUp = FALSE;
//fprintf(stdout,"nach setcaption\n");
//    kmand->SetClientSize(500, 375);// Initial-Size
//fprintf(stdout,"nach setclientsize\n");

    //fprintf(stdout,"nach show()\n");
    QString S=klocale->language();
fprintf(stdout,"%s\n",S.data());
fprintf(stdout,"%s\n",klocale->translate("This is a test-output"));
    Res = MApp->exec();
   return Res;
}