#ifndef MANDEL_M
#define MANDEL_M
#include <kstatusbar.h>
#include <kiconloader.h> 
#include <ktoolbar.h>
#include "fracglobals.h"

#define ZOOM_NONE 0       // kein Zooming
#define ZOOM_NORM 1       // normales Zooming
#define ZOOM_CLICK 2      // Point & Click
#define ZOOM_MOVE 3       // Rectangle moving, Fractint-compatible
#define ZOOM_ANIMATION 4  // Animation-Zoom, automatic calculation

// Konstanten fr die Statuszeile
#define SZ_TYPE  0        // Feld Fraktaltyp
#define SZ_PROGRESS 1     // Fortschrittsanzeige

class DrawWidget : public QWindow
{
   Q_OBJECT
public:
   DrawWidget( QWidget* parent=0, const char* name=0, WFlags f=0 );
   void SetPicSize(int SizeX, int SizeY);  // Gre setzen
   int GetSizeX(void) {return PicSizeX; }        // x-Gre lesen
   int GetSizeY(void) {return PicSizeY; }        // y-Gre lesen
   void AbortCalc(void);           // Berechnung abbrechen
   void NewFractal(int FracType);  // neues Fraktal(-typ) berechnen
   int LoadColorMap(char* MapName);
   int SaveColorMap(void);         // Farbpalette speichern
   int SelectColorMap(void);       // Palette laden
   void ChangePicParams(int sx, int sy, int cdeep); // Bildparameter ndern
   void EditColorMap(void);        // Palette editieren
   void CalcFractal(void);         // berechnen, fr alle Typen
   void CalcMandel(void);          // berechnen, nur Field-Types
   void CalcOrbit(void);           //     "    , nur Orbits
   void InitPalette(void);         // Farbpalette initialisieren
   void LoadPic(char* DateiName);  // Image laden
   void GetParamsBlock(struct MandDataType* MandData); // Daten fr speichern
   void SetParamsBlock(struct MandDataType* MandData); // Daten aus Datei setzen
   void PutPixel(int x, int y, long Iter);  
   long GetPixel(int x, int y);  
   void PicToColorMap(void);       // Farbpalette aus dem gelesenen Bild 

   BaseFract * Fractal;            // das Fraktal
   int ColorDeep;                  // Farbtiefe 1,8,32
   unsigned long NumColors;        // Anzahl der Farben
   double ZoomFactor;              // Zoomfaktor normal (Zoomin/ZoomOut)
   double ZoomFactorAnimation;     // Zoomfaktor Animationszoom
   int ZoomAnimationPics;          // # der Bilder pro Anim-Zoom
   int SaveColorMapBool;           // Farbpalette mit dem Parameterfile speichern
   int UpdateInterval;             // Lines zwischen zwei Screenupdates
   int UpdateLevel;                // UI fr Rechenmode "Rectangle"
   int UpdateBool;                 // Anzeige whrend der Berechnung (J/N)
   int HiColorMode;                // Farbgebung im Hicolor-Mode
   int CalcMethod;                 // Rechenmethode
   
   int Berechnung;           // TRUE=Berechnung luft, FALSE keine Berechnung
   int ImageReady;           // Das Bild ist fertig und kann direkt dargestellt werden

   unsigned long ColorMap[256];    // Farbpalette
protected:
   void paintEvent( QPaintEvent * );   
   void resizeEvent( QResizeEvent * );
   void mousePressEvent (QMouseEvent*);
   void mouseReleaseEvent (QMouseEvent*);
   //  void mouseDoubleClickEvent (QMouseEvent*);
   void mouseMoveEvent (QMouseEvent*);
signals:
   void FertigSig();         // Berechnung ist fertig
//   void Start();             // Berechnung starten
public slots:
   void MenuZoomIn();
   void MenuZoomOut();
   void MenuSavePic();       // Bild speichern
   void Fertig();            // Aktion bei Berechnung fertig
private:
   QImage * Image;           // Puffer fr berechnetes Bild
   QPixmap * Pix;
   QPainter * Paint;
   int PicSizeX,PicSizeY;    // Gre des Bildes in Pixeln
   int Resized;              // Die Bildgre wurde gendert, neu berechnen...

   int ZoomMode;             // Modus fr Zooming
   int ZoomX1,ZoomY1;        // Koordinaten des 1. Punktes
   int ZoomX2,ZoomY2;        // Koordinaten des 2. Punktes
   int zxa,zya,bra,hoa;      // zum Lschen des Rechtecks
   long PixCount,ModVal;     // # der Pickel, fr Status
   long PixMax;              // Gesamtanzahl der Pickel
   
   
   void PaletteInterpolate(int Start, int Ende); // Farbverlauf
   void CreatePalette(void);
   int TestRect (int x1, int x2, int y1, int y2);
   void FillRect(int x1, int x2, int y1, int y2, int Level);
   void Calc2w  (int x1, int x2, int y1, int y2, int Level);
   void Calc2s  (int x1, int x2, int y1, int y2, int Level);
   void Calc2(void);
};


class KMandel : public KTopLevelWidget
{
   Q_OBJECT
public:
    KMandel(const char *name=0 );
    ~KMandel();
    void SetClientSize(int Breite, int Hoehe);
    void SetStatus(char* Text, int ID); // Text in der Statuszeile ndern
    DrawWidget *Client;  // Clientbereich
    KStatusBar* StatusZeile;
protected:
    void resizeEvent( QResizeEvent * );
private slots:
   void MenuNew();                    // Menpunkt "New"
   void MenuOpen();                   // Datei lesen
   void MenuWriteParams();            // Faktalparameter speichern
   void MenuFileSettings();           // Dateiparameter einstellen
   void ShowFracTypeDlg();            // Fraktaltpen-Dialog
   void ShowFracParamsDlg();          // Fraktalparameter (allgemein)
   void MenuSpecialParams();          // Typ-spezifische Einstellung
   void MenuPicSettings();            // generelle Bildeinstellung
   void MenuEditPal();                // Farbpalette bearbeiten
   void MenuLoadColorMap();           // Farbpalette laden
   void MenuSaveColorMap();           // Farbpalette speichern
   void MenuZoomSettings();           // Einstellungen Zoom
   void ShowAboutDlg();
   void MenuHelp();                   // Menpunkt Hilfe
private:
   KMenuBar* MainMenu;
   KToolBar* ToolBar;
   KConfig* Config;
   void InitMenue();
   void InitStatusZeile();
   void InitToolBar(void);
   void InitParameterRec(struct MandDataType* MD);// Parameterblock (typunabhngig) initialisieren
   void SetKeys(void);
   int  LoadParamFile(QString* DN);
};

#endif
