// Fraktaltyp Mandel hoch N
// p[0] = pixel.x
// p[1] = pixel.y
#include <stdio.h>
#include <stdlib.h>
#include "mandel_n.h"
#include "mandel_n.moc"

double MandelHochNPar[2] = {0, 0.0};

void MandelHochN::Init(int SizeX, int SizeY)
{
int i;

   BaseFract::Init(SizeX, SizeY);
   SetDefaultCorners(&li,&re,&un,&ob);
   for (i=0; i<16;i++) p[i]=SetDefaultParams(i);
   l[0] = 2;
   SetSize(SizeX, SizeY);
}

void MandelHochN::SetDefaultCorners(double *x1, double *x2, double *y1, double *y2)
{
   *x1 = -2.0;     // Gre des Fraktals
   *x2 =  2.0;
   *y1 = -1.5;
   *y2 =  1.5;
}

double MandelHochN::SetDefaultParams(int Index)
{
   if (Index<0) return 0.0;
   if (Index >= 2) return 0.0;
   return MandelHochNPar[Index];
}

int MandelHochN::CalcPoint(int x, int y)
{
double c;
double rx = li+dx*x;
double ry = un+dy*y;
double a=rx+p[0];
double b=ry+p[1];
double aa=a, bb=b;
int    iter=0;
int Lauf;

   do {
      aa=a;   bb=b;
      for (Lauf=0; Lauf < l[0]; Lauf++) {
         c = a*aa - b*bb;
         b = a*bb+b*aa;
         a = c;
      }
      a += rx;
      b += ry;
      iter++;
   } while ((iter<MaxIter) && ConvTest(a*a,b*b) );

   return PostCompute(iter,a,b);
}

char* MandelHochN::GetFracInfo(void)
{
   return "Formula: x=c=Pixel;\n     loop {x=x^N+c};\n     until (x^2>Bail)";
}

int MandelHochN::SetupFrac(BaseFract* Frac) // type specific-setup
{
MandelHochNDlg MDlg = MandelHochNDlg(Frac, 0, "Mandel hoch N settings");

   if (MDlg.exec()) {
      return MDlg.SetSpecificParams();
   } else
      return FALSE;
}


// -----------------------------------------

MandelHochNDlg::MandelHochNDlg(BaseFract* Frac, QWidget* parent=0, const char* name=0):
          QDialog(parent,name, WStyle_DialogBorder)
{
char Strg[32];
   Fractal = Frac;
   setFixedSize(300,200);
   setCaption(Fractal->GetFracName());

   QLabel *Label0 = new QLabel("Offset values of x(0)", this, "Label0");
   Label0->setGeometry(30,20,150,20);

   ConstX = new QLineEdit(this, "OffsetX");
   ConstX->setGeometry(100,50,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[0]);
   ConstX->setText(Strg);
   ConstY = new QLineEdit(this, "OffsetY");
   ConstY->setGeometry(100,80,125,22);
   sprintf(Strg,"%0.14f", Fractal->p[1]);
   ConstY->setText(Strg);

   QLabel* Label1 = new QLabel("OffsetX",this,"Label1");
   Label1->setGeometry(20,50,75,20);
   QLabel* Label2 = new QLabel("OffsetY",this,"Label2");
   Label2->setGeometry(20,80,75,20);

   QLabel* OrderLab = new QLabel("Order",this,"OrderLab");
   OrderLab->setGeometry(20,110,50,20);
   
   PotSpin = new KNumericSpinBox(this, "PotSpin");
   PotSpin->setGeometry(100,110,50,20);
   PotSpin->setRange(1,100);
   PotSpin->setEditable(TRUE);
   PotSpin->setValue(Fractal->l[0]);

   QPushButton* OkBtn = new QPushButton("OK", this, "OkBtn");
   OkBtn->setGeometry(20,150,80,30);
   OkBtn->setDefault(TRUE);
   connect(OkBtn,SIGNAL(clicked()), this, SLOT(accept()) );

   QPushButton* DefaultsBtn = new QPushButton("Defaults", this, "DefautsBtn");
   DefaultsBtn->setGeometry(110,150,80,30);
   connect(DefaultsBtn,SIGNAL(clicked()), this, SLOT(SetDefaults()) );

   QPushButton* CancelBtn = new QPushButton("Cancel", this, "OkBtn");
   CancelBtn->setGeometry(200,150,80,30);
   connect(CancelBtn,SIGNAL(clicked()), this, SLOT(reject()) );
}

int MandelHochNDlg::SetSpecificParams(void)
{
char* eptr;
double p0,p1;
long l0;

   p0 = strtod(ConstX->text(), &eptr);
   if (*eptr!=0) return FALSE;

   p1 = strtod(ConstY->text(), &eptr);
   if (*eptr!=0) return FALSE;

   l0 = PotSpin->getValue();

   Fractal->p[0] = p0;
   Fractal->p[1] = p1;
   Fractal->l[0] = l0;
   return TRUE;
}

void MandelHochNDlg::SetDefaults()
{
char Strg[32];

   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(0));
   ConstX->setText(Strg);
   sprintf(Strg,"%0.14f", Fractal->SetDefaultParams(1));
   ConstY->setText(Strg);
   PotSpin->setValue(2);
}